# -*- coding: utf-8 -*-
"""Version code adopted from Django development version.

https://github.com/django/django
"""
from __future__ import unicode_literals

import datetime
import os
import subprocess


default_app_config = 'gum.apps.GumConfig'
VERSION = (1, 1, 1, 'final', 0)


def get_version(version=None):
    """Returns a PEP 386-compliant version number from VERSION.
    :param version:
    """
    if version is None:
        from gum import VERSION as version
    else:
        assert len(version) == 5
        assert version[3] in ('alpha', 'beta', 'rc', 'final')

    # Now build the two parts of the version number:
    # main = X.Y[.Z]
    # sub = .devN - for pre-alpha releases
    #     | {a|b|c}N - for alpha, beta and rc releases

    parts = 2 if version[2] == 0 else 3
    main = '.'.join(str(x) for x in version[:parts])

    sub = ''
    if version[3] == 'alpha' and version[4] == 0:
        git_changeset = get_git_changeset()
        if git_changeset:
            sub = '.dev%s' % git_changeset

    elif version[3] != 'final':
        mapping = {'alpha': 'a', 'beta': 'b', 'rc': 'c'}
        sub = mapping[version[3]] + str(version[4])

    return str(main + sub)


def get_git_changeset():
    """Returns a numeric identifier of the latest git changeset.

    The result is the UTC timestamp of the changeset in YYYYMMDDHHMMSS format.
    This value isn't guaranteed to be unique, but collisions are very unlikely,
    so it's sufficient for generating the development version numbers.
    """
    repo_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    git_log = subprocess.Popen('git log --pretty=format:%ct --quiet -1 HEAD',
                               stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                               shell=True, cwd=repo_dir, universal_newlines=True)
    timestamp = git_log.communicate()[0]
    try:
        timestamp = datetime.datetime.utcfromtimestamp(int(timestamp))
    except ValueError:
        return None
    return timestamp.strftime('%Y%m%d%H%M%S')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from gum.utils import elasticsearch_connection


class ElasticsearchManager(object):
    """Like a `ModelManager` gives to the user methods to apply queries
    to Elasticsearch from a specific model.
    """

    def __init__(self, model=None, mapping_type=None, urls=None):
        self.model = model
        self.mapping_type = mapping_type
        self.urls = urls

    def get_elasticsearch_connection(self):
        """Gets the Elasticsearch connection with the urls attribute"""
        if self.mapping_type is not None:
            return self.mapping_type.get_elasticsearch_connection()
        return elasticsearch_connection(urls=self.urls)

    def search(self, **kwargs):
        """Partial application of `search` function from Elasticsearch
        module.

        :param kwargs:
        """
        es = self.get_elasticsearch_connection()
        if 'index' not in kwargs:
            kwargs["index"] = self.mapping_type.index
        if 'doc_type' not in kwargs:
            kwargs["doc_type"] = self.mapping_type.get_type()
        return es.search(**kwargs)

    def index(self, instance):
        """Shortcut to index an instance.

        :param instance:
        :return:
        """
        return self.mapping_type.index_document(instance)


class GenericElasticsearchManager(ElasticsearchManager):
    """Generic Elasticsearch manager to make queries without using MappingTypes."""

    def search(self, **kwargs):
        """For this manager it's mandatory to specify index and doc_type on each call:

        >>> elasticsearch = GenericElasticsearchManager()
        >>> elasticsearch.search(index="index-name", doc_type="mapping-type-name")

        :param kwargs:
        :return:
        """
        assert "index" in kwargs
        assert "doc_type" in kwargs
        return super(GenericElasticsearchManager, self).search(**kwargs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import logging
from celery.task import task
from celery.utils.log import get_task_logger
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ObjectDoesNotExist


logger = get_task_logger(__name__)


@task
def handle_save(sender_content_type_pk, instance_pk):
    """Async task to handle the indexation of a model.
    :param instance_pk:
    :param sender_content_type_pk:
    """
    from gum.indexer import indexer, NotRegistered
    try:
        sender_content_type = ContentType.objects.get(pk=sender_content_type_pk)
        sender = sender_content_type.model_class()
        instance = sender.objects.get(pk=instance_pk)
    except ObjectDoesNotExist:
        logger.warning("Object ({}, {}) not found".format(sender_content_type_pk, instance_pk))
        return False
    try:
        mapping_type = indexer.get_mapping_type(sender)
        mapping_type.index_document(instance)
    except NotRegistered:
        logger.warning("Object ({}, {}) not register".format(sender_content_type_pk, instance_pk))
        return False
    return True


@task
def handle_delete(sender_content_type_pk, instance_pk):
    """Async task to delete a model from the index.
    :param instance_pk:
    :param sender_content_type_pk:
    """
    from gum.indexer import indexer, NotRegistered
    try:
        sender_content_type = ContentType.objects.get(pk=sender_content_type_pk)
        sender = sender_content_type.model_class()
        instance = sender.objects.get(pk=instance_pk)
    except ObjectDoesNotExist:
        logger.warning("Object ({}, {}) not found".format(sender_content_type_pk, instance_pk))
        return False
    try:
        mapping_type = indexer.get_mapping_type(sender)
        mapping_type.delete_document(instance)
    except NotRegistered:
        return False
    return True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.apps import AppConfig


class GumConfig(AppConfig):
    name = 'gum'
    verbose_name = 'Gum'

    def ready(self):
        from gum.models import handle_indexer_registrations
        handle_indexer_registrations()
